/*
 * File: parser.cpp
 *
 * Written by Peter Horak in 2012 at Dartmouth College for the GreenCube project,
 * supervised by Professor Kristina Lynch.
 *
 * Purpose: parse binary data from the payload's (Bob's) SD card.
 */

#include <iostream>
#include <string>
#include <sstream>

#include <stdint.h>

using namespace std;

// For holding parsed data lines while waiting for the end of a GPS string
stringstream data_buffer;

// For holding partial GPS strings
stringstream gps_buffer;

// Used to find the start of each data group
char data_sentinel = '#';
char gps_sentinel = '$';

// Number of characters to expect in a GPS line
unsigned int gps_string_length = 30;

int main(int argc, char **argv)
{
    // To hold the data
    uint32_t time; // 4 bytes (unsigned int)
    int16_t data[10]; // 2 bytes each (short)
    char c; // 1 byte

    // Convert the data into text and output it to the data buffer
    cin.read((&c),sizeof(c));
    do {
        // Find the start of a line
        while ((c != data_sentinel) && (c != gps_sentinel))
            cin.read((&c),sizeof(c));
        // Parse a data line
        if (c == data_sentinel)
        {
            cin.read(reinterpret_cast<char *>(&time),sizeof(time));
            data_buffer << time << ' ';
            for (int i = 0; i < 10; i++)
            {
                cin.read(reinterpret_cast<char *>(&data[i]),sizeof(data[i]));
                data_buffer << data[i] << ' ';
            }
            data_buffer << endl;
            // Update c so it is no longer the previous sentinel
            cin.read((&c),sizeof(c));
        }
        else // Hangle parts of GPS strings
        {
            cin.read((&c),sizeof(c));
            if (c == gps_sentinel) // The start of a new string ($$)
            {
                // Output the previous GPS string and subsequent data
                cout << gps_buffer.str() << endl;
                cout << data_buffer.str();
                // Clear the buffers
                gps_buffer.str("");
                data_buffer.str("");
            }
            // Read part of a GPS string into the GPS buffer
            // A sentinel indicates either a new GPS string or a data line
            while (c != data_sentinel && c != gps_sentinel)
            {
                gps_buffer << c;
                cin.read((&c),sizeof(c));
            }
        }
    } while (cin && !cin.eof());
}
