/*
 * File: serial.c
 *
 * Modified from Tim Smith's serial code by Peter Horak.
 * Purpose: recieve (binary) serial data from the Arduino
 */

#include <stdio.h>   /* Standard input/output definitions */
#include <string.h>  /* For strlen */
#include <stdlib.h>  /* For calloc */
#include <unistd.h>  /* UNIX standard function definitions */
#include <fcntl.h>   /* File control definitions */
#include <errno.h>   /* Error number definitions */
#include <termios.h> /* POSIX terminal control definitions */
#include <dirent.h>  /* read directories and devices */
#include <signal.h>  /* For catching ^C interrupt */
#include <sys/time.h>/* For gettimeofday() */

int main(int argc, char *argv[]){

    char devPort[100]; // Serial port to talk through
    char outFile[256]; // Output file name
    FILE *fp; // Pointer to the output file

    if (argc < 2)
    {
        fprintf(stderr,"Usage: must specify the device to connect to.\n");
        usleep(1000000);
        return 1; // failure
    }
    sprintf(devPort,"/dev/%s",argv[1]);
    fprintf(stderr,"Connecting to serial port \"%s\"\n",devPort);
    int fd;
    fd = open(devPort,(O_RDONLY | O_NOCTTY | O_NDELAY));
    if (fd < 0)
    {
        fprintf(stderr,"Failed to connect to device.\n");
        usleep(1000000);
        return 1;
    }
    fcntl(fd, F_SETFL, 0);

    //init port==================
    struct termios options;
    // Get the current options for the port...
    tcgetattr(fd, &options);
    // Set the baud rate
    cfsetispeed(&options, B230400);
    cfsetospeed(&options, B230400);
//    cfsetispeed(&options, B115200);
//    cfsetospeed(&options, B115200);

    // Enable the receiver and set local mode...
    options.c_cflag |= (CLOCAL | CREAD);
    options.c_cflag &= ~(CSIZE | PARENB | CSTOPB);
    options.c_cflag |= CS8;
    // Set the new options for the port...
    tcsetattr(fd, TCSANOW, &options);
    //  end init port==================
    
    // open output file with unique name
    time_t mytime;
    struct tm *ltime;
    mytime = time(NULL);
    ltime = localtime(&mytime);
    sprintf(outFile, "data/GreenCubeDNT_%02d-%02d-%04d_%02d:%02d:%02d.dat",
            ltime->tm_mon+1,  ltime->tm_mday, ltime->tm_year+1900,
            ltime->tm_hour, ltime->tm_min,  ltime->tm_sec);
    if (NULL == (fp = fopen(outFile,"w")))
    {
        fprintf(stderr,"Failed to open output file.\n");
        usleep(1000000);
        return 1;
    }
    fprintf(stderr,"Writing data to \"%s\"\n",outFile);
    
    char result[2];
    int iRawCh;
    fcntl(fd, F_SETFL, FNDELAY); // don't block serial read
    while(1){
        iRawCh = read(fd, result, 1);
        if (iRawCh>0)
        {
            fprintf(stdout, "%c", result[0]);
            fprintf(fp, "%c", result[0]);
            fflush(stdout);
            fflush(fp);
        }
        else 
            usleep(5000);
    }
    close(fd);
    closef(fp);
    return 0; // success
}
